from airflow.providers.common.sql.sensors.sql import SqlSensor

from airflow_clickhouse_plugin.hooks.clickhouse_dbapi import ClickHouseDbApiHook
from airflow_clickhouse_plugin.operators.clickhouse_dbapi import \
    ClickHouseDbApiHookMixin


class ClickHouseSqlSensor(ClickHouseDbApiHookMixin, SqlSensor):
    def _get_hook(self) -> ClickHouseDbApiHook:
        return self._get_clickhouse_db_api_hook()



import typing as t

from airflow.providers.common.sql.operators import sql

from airflow_clickhouse_plugin.hooks.clickhouse_dbapi import \
    ClickHouseDbApiHook


class ClickHouseDbApiHookMixin(object):
    # these attributes are defined in both BaseSQLOperator and SqlSensor
    conn_id: str
    hook_params: t.Optional[dict]

    def _get_clickhouse_db_api_hook(self, **extra_hook_params) -> ClickHouseDbApiHook:
        hook_kwargs = {}
        if self.conn_id is not None:
            hook_kwargs['clickhouse_conn_id'] = self.conn_id
        if self.hook_params is not None:
            hook_kwargs.update(self.hook_params)
        hook_kwargs.update(extra_hook_params)
        return ClickHouseDbApiHook(**hook_kwargs)


class ClickHouseBaseDbApiOperator(ClickHouseDbApiHookMixin, sql.BaseSQLOperator):
    def get_db_hook(self) -> ClickHouseDbApiHook:
        return self._get_clickhouse_db_api_hook(schema=self.database)


class ClickHouseSQLExecuteQueryOperator(
    ClickHouseBaseDbApiOperator,
    sql.SQLExecuteQueryOperator,
):
    pass


class ClickHouseSQLColumnCheckOperator(
    ClickHouseBaseDbApiOperator,
    sql.SQLColumnCheckOperator,
):
    pass


class ClickHouseSQLTableCheckOperator(
    ClickHouseBaseDbApiOperator,
    sql.SQLTableCheckOperator,
):
    pass


class ClickHouseSQLCheckOperator(
    ClickHouseBaseDbApiOperator,
    sql.SQLCheckOperator,
):
    pass


class ClickHouseSQLValueCheckOperator(
    ClickHouseBaseDbApiOperator,
    sql.SQLValueCheckOperator,
):
    pass


class ClickHouseSQLIntervalCheckOperator(
    ClickHouseBaseDbApiOperator,
    sql.SQLIntervalCheckOperator,
):
    pass


class ClickHouseSQLThresholdCheckOperator(
    ClickHouseBaseDbApiOperator,
    sql.SQLThresholdCheckOperator,
):
    pass


class ClickHouseBranchSQLOperator(
    ClickHouseBaseDbApiOperator,
    sql.BranchSQLOperator,
):
    pass



import typing as t

from airflow.models import BaseOperator

from airflow_clickhouse_plugin.hooks.clickhouse import ClickHouseHook, \
    ExecuteParamsT, ExecuteReturnT, default_conn_name


class BaseClickHouseOperator(BaseOperator):
    """
    A superclass for operator classes. Defines __init__ with common arguments.

    Includes arguments of clickhouse_driver.Client.execute.
    """

    template_fields = (  # all str-containing arguments
        '_sql',
        '_parameters',
        '_external_tables',
        '_query_id',
        '_settings',
        '_database',
    )
    template_ext: t.Sequence[str] = ('.sql',)
    template_fields_renderers = {
        '_sql': 'sql',
        '_parameters': 'json',
        '_external_tables': 'json',
        '_settings': 'json',
    }

    def __init__(
            self,
            *args,
            sql: t.Union[str, t.Iterable[str]],
            # arguments of clickhouse_driver.Client.execute
            parameters: t.Optional[ExecuteParamsT] = None,
            with_column_types: bool = False,
            external_tables: t.Optional[t.List[dict]] = None,
            query_id: t.Optional[str] = None,
            settings: t.Dict[str, t.Any] = None,
            types_check: bool = False,
            columnar: bool = False,
            # arguments of ClickHouseHook.__init__
            clickhouse_conn_id: str = default_conn_name,
            database: t.Optional[str] = None,
            **kwargs,
    ):
        super().__init__(*args, **kwargs)

        self._sql = sql

        self._parameters = parameters
        self._with_column_types = with_column_types
        self._external_tables = external_tables
        self._query_id = query_id
        self._settings = settings
        self._types_check = types_check
        self._columnar = columnar

        self._clickhouse_conn_id = clickhouse_conn_id
        self._database = database

    def _hook_execute(self) -> ExecuteReturnT:
        hook = ClickHouseHook(
            clickhouse_conn_id=self._clickhouse_conn_id,
            database=self._database,
        )
        return hook.execute(
            self._sql,
            self._parameters,
            self._with_column_types,
            self._external_tables,
            self._query_id,
            self._settings,
            self._types_check,
            self._columnar,
        )


class ClickHouseOperator(BaseClickHouseOperator, BaseOperator):
    def execute(self, context: t.Dict[str, t.Any]) -> ExecuteReturnT:
        return self._hook_execute()



import typing as t

from airflow.exceptions import AirflowException
from airflow.sensors.base import BaseSensorOperator

from airflow_clickhouse_plugin.hooks.clickhouse import ExecuteReturnT
from airflow_clickhouse_plugin.operators.clickhouse import \
    BaseClickHouseOperator


class ClickHouseSensor(BaseClickHouseOperator, BaseSensorOperator):
    """ Pokes using clickhouse_driver.Client.execute. """

    def __init__(
            self,
            *args,
            is_failure: t.Callable[[ExecuteReturnT], bool] = None,
            is_success: t.Callable[[ExecuteReturnT], bool] = None,
            **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self._is_failure = is_failure
        self._is_success = bool if is_success is None else is_success

    def poke(self, context: dict) -> bool:
        result = self._hook_execute()
        if self._is_failure is not None:
            is_failure = self._is_failure(result)
            if is_failure:
                raise AirflowException(f'is_failure returned {is_failure}')
        return self._is_success(result)
